# -*- coding: utf-8 -*-

''' Extends the ``unittest2.TestCase`` to so get better string difference.

It will show the string difference on RED color.
'''

from unittest2 import TestCase
from termcolor import colored


class UtilsTestCase(TestCase):
    ''' Adds a custom method to get a better output when checking
    for long strings
    '''

    # the number of chars of the strings that should be colorized, when
    # there is a difference
    COLORED_CHARS = 5

    # the total number of chars that should be shown from each string
    # to get the difference (from the left and from the right)
    TOTAL_CHARS = 25

    # the color in which the difference will be shown.
    DIFFERENCE_COLOR = 'red'

    def assertEqual(self, first, second, msg=None):
        ''' Extends the assertEqualto get a better output on the string
        difference when both values are strings (or unicode)
        '''

        if isinstance(first, basestring) and isinstance(second, basestring):
            # check if the string are multine. In that case use the
            # default solution
            if not '\n' in first and not '\n' in second:
                if first != second:
                    # if the aren't multi line, then use the custom string
                    # difference report
                    standardMsg = self._search_string_difference(first, second)
                    msg = self._formatMessage(msg, standardMsg)
                    raise self.failureException(msg)
            else:
                return super(UtilsTestCase, self).assertEqual(first, second,
                                                              msg)

        return super(UtilsTestCase, self).assertEqual(first, second, msg)

    # override the assertEquals
    assertEquals = assertEqual

    def _search_string_difference(self, first, second):
        ''' Given two strings it searches for the string difference on
        a readable manner.
        '''
        index = 0
        min_length = min(len(first), len(second))
        res = ""
        while index < min_length:
            if first[index] == second[index]:
                index += 1
                continue

            res = "'%s' != '%s'" % (
                            self._show_string_difference(first, index),
                            self._show_string_difference(second, index)
                            )
            break

        if not res:
            # entonces uno es mas largo que otro
            shortest = first
            longest = second
            if len(first) > len(second):
                longest = first
                shortest = second

            res = "'%s' != '%s'" % (
                            self._show_string_difference(shortest, index),
                            self._show_string_difference(longest, index)
                            )

        return res

    def _show_string_difference(self, value, index):
        res = value[index-self.TOTAL_CHARS: index-self.COLORED_CHARS]
        res += colored(value[index-self.COLORED_CHARS:index+self.COLORED_CHARS],
                        self.DIFFERENCE_COLOR,
                        attrs=['bold'])
        res += value[index+self.COLORED_CHARS:index+self.TOTAL_CHARS]
        return res



# -*- coding: utf-8 -*-

''' Outputs with color the head of each test which failed.
'''

from termcolor import colored
from unittest2.runner import TextTestResult


class UtilsResult(TextTestResult):
    ''' A custom result which will print the result using colored
    output
    '''

    def color_favour(self, flavour, test):
        msg = '%s: %s' % (flavour, self.getDescription(test))
        return colored(msg, 'red', attrs=['bold'])

    def printErrorList(self, flavour, errors):
        for test, err in errors:
            self.stream.writeln(self.separator1)
            self.stream.writeln(self.color_favour(flavour, test))
            self.stream.writeln(self.separator2)
            self.stream.writeln("%s" % err)



# -*- coding: utf-8 -*-

'''
'''

from unittest2.runner import TextTestRunner
from unittest_utils.result import UtilsResult


class UtilsTextTestRunner(TextTestRunner):
    ''' Uses the result that prints the failed an error with color.
    '''

    resultclass = UtilsResult



# -*- coding: utf-8 -*-

''' Extends the custom loader so the test are run by the line
where they are defined and not by thier names.
'''

from unittest2.loader import TestLoader


def sortResultsByLineNumber(testCaseClass, fnName):
    ''' Given a function name of the testCaseClass, return the line
    number of that function
    '''
    return getattr(testCaseClass, fnName).im_func.func_code.co_firstlineno


class UtilsTestLoader(TestLoader):
    ''' Custom loader that will execute the test for a module using
    the line number where the method starts and not the method name.
    '''

    def getTestCaseNames(self, testCaseClass):
        """Return a sorted sequence of method names found within testCaseClass.

        The sequence will be returned by the line number
        """
        testFnNames = super(UtilsTestLoader, self).getTestCaseNames(testCaseClass)
        testFnNames.sort(key=lambda fnName: sortResultsByLineNumber(testCaseClass, fnName))
        return testFnNames




from setuptools import setup, find_packages


setup(
    name='unittest-utils',
    version='0.1.1',
    description='Some custom utils for unittest2',
    long_description=open('README.rst').read(-1),
    author='',
    author_email='',
    url='',
    install_requires=[
        "unittest2",
        "termcolor",
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts':
            ['unittest-utils=unittest_utils.main:_main']
    }
)



# -*- coding: utf-8 -*-

''' Extends the ``unittest2.main.TestProgram`` to add the custom options.
'''


from unittest2.main import TestProgram, USAGE_AS_MAIN
from unittest_utils.loader import UtilsTestLoader
from unittest_utils.runner import UtilsTextTestRunner


class UtilsTestProgram(TestProgram):

    def __init__(self, *args, **kwargs):
        ''' Overrides de default TestProgram to use a custom testLoader
        which will load the tests using the line where those are defined.
        '''
        super(UtilsTestProgram, self).__init__(testLoader=UtilsTestLoader(),
                                               testRunner=UtilsTextTestRunner,
                                                *args,
                                                **kwargs)

    def _do_discovery(self, *args, **kwargs):
        return super(UtilsTestProgram, self)._do_discovery(Loader=UtilsTestLoader,
                                                           *args,
                                                           **kwargs)


def _main():
    UtilsTestProgram.USAGE = USAGE_AS_MAIN
    UtilsTestProgram(module=None)
